# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SalesTaxCodeListParams"]


class SalesTaxCodeListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    ids: List[str]
    """
    Filter for specific sales-tax codes by their QuickBooks-assigned unique
    identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    limit: int
    """The maximum number of objects to return.

    **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
    sales-tax codes. This parameter will limit the response size, but you cannot
    fetch subsequent results using a cursor. For pagination, use the name-range
    parameters instead (e.g., `nameFrom=A&nameTo=B`).

    When this parameter is omitted, the endpoint returns all sales-tax codes without
    limit, unlike paginated endpoints which default to 150 records. This is
    acceptable because sales-tax codes typically have low record counts.
    """

    name_contains: Annotated[str, PropertyInfo(alias="nameContains")]
    """
    Filter for sales-tax codes whose `name` contains this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameStartsWith` or `nameEndsWith`.
    """

    name_ends_with: Annotated[str, PropertyInfo(alias="nameEndsWith")]
    """
    Filter for sales-tax codes whose `name` ends with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameStartsWith`.
    """

    name_from: Annotated[str, PropertyInfo(alias="nameFrom")]
    """
    Filter for sales-tax codes whose `name` is alphabetically greater than or equal
    to this value.
    """

    names: List[str]
    """Filter for specific sales-tax codes by their name(s), case-insensitive.

    Like `id`, `name` is a unique identifier for a sales-tax code.

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    name_starts_with: Annotated[str, PropertyInfo(alias="nameStartsWith")]
    """
    Filter for sales-tax codes whose `name` starts with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameEndsWith`.
    """

    name_to: Annotated[str, PropertyInfo(alias="nameTo")]
    """
    Filter for sales-tax codes whose `name` is alphabetically less than or equal to
    this value.
    """

    status: Literal["active", "all", "inactive"]
    """Filter for sales-tax codes that are active, inactive, or both."""

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for sales-tax codes updated on or after this date and time, in ISO 8601
    format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
    is assumed to be 00:00:00 of that day.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for sales-tax codes updated on or before this date and time, in ISO 8601
    format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
    is assumed to be 23:59:59 of that day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "EmployeeCreateParams",
    "AdditionalNote",
    "Address",
    "CustomContactField",
    "EmergencyContact",
    "EmergencyContactPrimaryContact",
    "EmergencyContactSecondaryContact",
    "EmployeePayroll",
    "EmployeePayrollEarning",
    "EmployeePayrollSickHours",
    "EmployeePayrollVacationHours",
]


class EmployeeCreateParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    account_number: Annotated[str, PropertyInfo(alias="accountNumber")]
    """
    The employee's account number, which appears in the QuickBooks chart of
    accounts, reports, and graphs.

    Note that if the "Use Account Numbers" preference is turned off in QuickBooks,
    the account number may not be visible in the user interface, but it can still be
    set and retrieved through the API.
    """

    additional_notes: Annotated[Iterable[AdditionalNote], PropertyInfo(alias="additionalNotes")]
    """Additional notes about this employee."""

    address: Address
    """The employee's address.

    If the company uses QuickBooks Payroll for this employee, this address must
    specify a complete address, including city, state, ZIP (or postal) code, and at
    least one line of the street address.
    """

    adjusted_service_date: Annotated[Union[str, date], PropertyInfo(alias="adjustedServiceDate", format="iso8601")]
    """The adjusted service date for this employee, in ISO 8601 format (YYYY-MM-DD).

    This date accounts for previous employment periods or leaves that affect
    seniority.
    """

    alternate_phone: Annotated[str, PropertyInfo(alias="alternatePhone")]
    """The employee's alternate telephone number."""

    billing_rate_id: Annotated[str, PropertyInfo(alias="billingRateId")]
    """
    The employee's billing rate, used to override service item rates in time
    tracking activities.
    """

    birth_date: Annotated[Union[str, date], PropertyInfo(alias="birthDate", format="iso8601")]
    """This employee's date of birth, in ISO 8601 format (YYYY-MM-DD)."""

    custom_contact_fields: Annotated[Iterable[CustomContactField], PropertyInfo(alias="customContactFields")]
    """
    Additional custom contact fields for this employee, such as phone numbers or
    email addresses.
    """

    department: str
    """The employee's department.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    description: str
    """A description of this employee.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    disability_description: Annotated[str, PropertyInfo(alias="disabilityDescription")]
    """A description of this employee's disability."""

    disability_status: Annotated[Literal["disabled", "non_disabled"], PropertyInfo(alias="disabilityStatus")]
    """Indicates whether this employee is disabled."""

    email: str
    """The employee's email address."""

    emergency_contact: Annotated[EmergencyContact, PropertyInfo(alias="emergencyContact")]
    """The employee's emergency contacts."""

    employee_payroll: Annotated[EmployeePayroll, PropertyInfo(alias="employeePayroll")]
    """The employee's payroll information."""

    employee_type: Annotated[Literal["officer", "owner", "regular", "statutory"], PropertyInfo(alias="employeeType")]
    """The employee type.

    This affects payroll taxes - a statutory employee is defined as an employee by
    statute. Note that owners/partners are typically on the "Other Names" list in
    QuickBooks, but if listed as an employee their type will be `owner`.
    """

    employment_status: Annotated[Literal["full_time", "part_time"], PropertyInfo(alias="employmentStatus")]
    """Indicates whether this employee is a part-time or full-time employee."""

    ethnicity: Literal["american_indian", "asian", "black", "hawaiian", "hispanic", "white", "two_or_more_races"]
    """This employee's ethnicity."""

    external_id: Annotated[str, PropertyInfo(alias="externalId")]
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.

    **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
    QuickBooks will return an error.
    """

    fax: str
    """The employee's fax number."""

    first_name: Annotated[str, PropertyInfo(alias="firstName")]
    """The employee's first name.

    Maximum length: 25 characters.
    """

    gender: Literal["male", "female"]
    """This employee's gender."""

    hired_date: Annotated[Union[str, date], PropertyInfo(alias="hiredDate", format="iso8601")]
    """The date this employee was hired, in ISO 8601 format (YYYY-MM-DD)."""

    i9_on_file_status: Annotated[Literal["on_file", "not_on_file"], PropertyInfo(alias="i9OnFileStatus")]
    """Indicates whether this employee's I-9 is on file."""

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this employee is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    job_title: Annotated[str, PropertyInfo(alias="jobTitle")]
    """The employee's job title."""

    key_employee_status: Annotated[Literal["key_employee", "non_key_employee"], PropertyInfo(alias="keyEmployeeStatus")]
    """Indicates whether this employee is a key employee."""

    last_name: Annotated[str, PropertyInfo(alias="lastName")]
    """The employee's last name.

    Maximum length: 25 characters.
    """

    middle_name: Annotated[str, PropertyInfo(alias="middleName")]
    """The employee's middle name.

    Maximum length: 5 characters.
    """

    military_status: Annotated[Literal["active", "reserve"], PropertyInfo(alias="militaryStatus")]
    """This employee's military status if they are a U.S. veteran."""

    mobile: str
    """The employee's mobile phone number."""

    note: str
    """A note or comment about this employee."""

    original_hire_date: Annotated[Union[str, date], PropertyInfo(alias="originalHireDate", format="iso8601")]
    """The original hire date for this employee, in ISO 8601 format (YYYY-MM-DD)."""

    overtime_exempt_status: Annotated[Literal["exempt", "non_exempt"], PropertyInfo(alias="overtimeExemptStatus")]
    """Indicates whether this employee is exempt from overtime pay.

    This classification is based on U.S. labor laws (FLSA).
    """

    pager: str
    """The employee's pager number."""

    pager_pin: Annotated[str, PropertyInfo(alias="pagerPin")]
    """The employee's pager PIN."""

    phone: str
    """The employee's primary telephone number."""

    print_as: Annotated[str, PropertyInfo(alias="printAs")]
    """The name to use when printing this employee from QuickBooks.

    By default, this is the same as the `name` field.
    """

    salutation: str
    """
    The employee's formal salutation title that precedes their name, such as "Mr.",
    "Ms.", or "Dr.".
    """

    ssn: str
    """The employee's Social Security Number. The value can be with or without dashes.

    **NOTE**: This field cannot be changed after the employee is created.
    """

    supervisor_id: Annotated[str, PropertyInfo(alias="supervisorId")]
    """The employee's supervisor.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    target_bonus: Annotated[str, PropertyInfo(alias="targetBonus")]
    """The target bonus for this employee, represented as a decimal string.

    Found in the "employment job details" section of the employee's record in
    QuickBooks.
    """

    termination_date: Annotated[Union[str, date], PropertyInfo(alias="terminationDate", format="iso8601")]
    """
    The date this employee's employment ended with the company, in ISO 8601 format
    (YYYY-MM-DD). This is also known as the released date or separation date.
    """

    us_citizenship_status: Annotated[Literal["citizen", "non_citizen"], PropertyInfo(alias="usCitizenshipStatus")]
    """Indicates whether this employee is a U.S. citizen."""

    us_veteran_status: Annotated[Literal["veteran", "non_veteran"], PropertyInfo(alias="usVeteranStatus")]
    """Indicates whether this employee is a U.S. veteran."""

    work_authorization_expiration_date: Annotated[
        Union[str, date], PropertyInfo(alias="workAuthorizationExpirationDate", format="iso8601")
    ]
    """
    The date this employee's work authorization expires, in ISO 8601 format
    (YYYY-MM-DD).
    """


class AdditionalNote(TypedDict, total=False):
    note: Required[str]
    """The text of this note."""


class Address(TypedDict, total=False):
    city: str
    """The city, district, suburb, town, or village name of the employee address.

    Maximum length: 31 characters.
    """

    country: str
    """The country name of the employee address."""

    line1: str
    """The first line of the employee address (e.g., street, PO Box, or company name).

    Maximum length: 41 characters.
    """

    line2: str
    """
    The second line of the employee address, if needed (e.g., apartment, suite,
    unit, or building).

    Maximum length: 41 characters.
    """

    line3: str
    """The third line of the employee address, if needed.

    Maximum length: 41 characters.
    """

    line4: str
    """The fourth line of the employee address, if needed.

    Maximum length: 41 characters.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The postal code or ZIP code of the employee address.

    Maximum length: 13 characters.
    """

    state: Literal[
        "none",
        "armed_forces_americas",
        "armed_forces_europe",
        "armed_forces_pacific",
        "AK",
        "AL",
        "AR",
        "AZ",
        "CA",
        "CO",
        "CT",
        "DC",
        "DE",
        "FL",
        "GA",
        "HI",
        "IA",
        "ID",
        "IL",
        "IN",
        "KS",
        "KY",
        "LA",
        "MA",
        "MD",
        "ME",
        "MI",
        "MN",
        "MO",
        "MS",
        "MT",
        "NB",
        "NC",
        "ND",
        "NE",
        "NH",
        "NJ",
        "NM",
        "NV",
        "NY",
        "OH",
        "OK",
        "OR",
        "PA",
        "PR",
        "RI",
        "SC",
        "SD",
        "TN",
        "TX",
        "UT",
        "VA",
        "VT",
        "WA",
        "WI",
        "WV",
        "WY",
    ]
    """The U.S.

    state of the employee address. QuickBooks requires this field to be a U.S. state
    abbreviation (e.g., "CA" for California). See enum for all possible values.
    """


class CustomContactField(TypedDict, total=False):
    name: Required[str]
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    value: Required[str]
    """The value of the contact field."""


class EmergencyContactPrimaryContact(TypedDict, total=False):
    name: Required[str]
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    value: Required[str]
    """The value of the contact field."""

    relation: Literal[
        "brother", "daughter", "father", "friend", "mother", "other", "partner", "sister", "son", "spouse"
    ]
    """The relationship of the employee to the employee."""


class EmergencyContactSecondaryContact(TypedDict, total=False):
    name: Required[str]
    """The name of the contact field (e.g., "old address", "secondary phone")."""

    value: Required[str]
    """The value of the contact field."""

    relation: Literal[
        "brother", "daughter", "father", "friend", "mother", "other", "partner", "sister", "son", "spouse"
    ]
    """The relationship of the employee to the employee."""


class EmergencyContact(TypedDict, total=False):
    primary_contact: Annotated[EmergencyContactPrimaryContact, PropertyInfo(alias="primaryContact")]
    """The employee's primary emergency contact."""

    secondary_contact: Annotated[EmergencyContactSecondaryContact, PropertyInfo(alias="secondaryContact")]
    """The employee's secondary emergency contact."""


class EmployeePayrollEarning(TypedDict, total=False):
    payroll_wage_item_id: Required[Annotated[str, PropertyInfo(alias="payrollWageItemId")]]
    """
    The payroll wage item that defines how this employee is paid (e.g., Regular Pay,
    Overtime Pay). This determines the payment scheme used for payroll calculations.
    """

    rate: str
    """The hourly rate for this employee, represented as a decimal string."""

    rate_percent: Annotated[str, PropertyInfo(alias="ratePercent")]
    """The hourly rate for this employee expressed as a percentage."""


class EmployeePayrollSickHours(TypedDict, total=False):
    accrual_period: Annotated[
        Literal["accrues_annually", "accrues_hourly", "accrues_per_paycheck"], PropertyInfo(alias="accrualPeriod")
    ]
    """How frequently the employee's sick hours are accrued."""

    accrual_start_date: Annotated[Union[str, date], PropertyInfo(alias="accrualStartDate", format="iso8601")]
    """
    The date the employee's sick hours began to accrue, in ISO 8601 format
    (YYYY-MM-DD).
    """

    hours_accrued_per_period: Annotated[str, PropertyInfo(alias="hoursAccruedPerPeriod")]
    """
    The number of sick hours the employee will accrue per accrual period, in ISO
    8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
    represented as PT1H30M.
    """

    hours_available: Annotated[str, PropertyInfo(alias="hoursAvailable")]
    """
    The total number of sick hours currently available for the employee to use, in
    ISO 8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30
    minutes is represented as PT1H30M. Defaults to 0.
    """

    hours_used: Annotated[str, PropertyInfo(alias="hoursUsed")]
    """
    The number of sick hours the employee has used, in ISO 8601 format for time
    intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    maximum_hours: Annotated[str, PropertyInfo(alias="maximumHours")]
    """
    The maximum number of sick hours the employee can accrue, in ISO 8601 format for
    time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    resets_hours_each_year: Annotated[bool, PropertyInfo(alias="resetsHoursEachYear")]
    """
    Indicates whether the employee's sick hours reset to zero at the beginning of
    the new year.
    """


class EmployeePayrollVacationHours(TypedDict, total=False):
    accrual_period: Annotated[
        Literal["accrues_annually", "accrues_hourly", "accrues_per_paycheck"], PropertyInfo(alias="accrualPeriod")
    ]
    """How frequently the employee's vacation hours are accrued."""

    accrual_start_date: Annotated[Union[str, date], PropertyInfo(alias="accrualStartDate", format="iso8601")]
    """
    The date the employee's vacation hours began to accrue, in ISO 8601 format
    (YYYY-MM-DD).
    """

    hours_accrued_per_period: Annotated[str, PropertyInfo(alias="hoursAccruedPerPeriod")]
    """
    The number of vacation hours the employee will accrue per accrual period, in ISO
    8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
    represented as PT1H30M.
    """

    hours_available: Annotated[str, PropertyInfo(alias="hoursAvailable")]
    """
    The total number of vacation hours currently available for the employee to use,
    in ISO 8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30
    minutes is represented as PT1H30M. Defaults to 0.
    """

    hours_used: Annotated[str, PropertyInfo(alias="hoursUsed")]
    """
    The number of vacation hours the employee has used, in ISO 8601 format for time
    intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented as
    PT1H30M.
    """

    maximum_hours: Annotated[str, PropertyInfo(alias="maximumHours")]
    """
    The maximum number of vacation hours the employee can accrue, in ISO 8601 format
    for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is represented
    as PT1H30M.
    """

    resets_hours_each_year: Annotated[bool, PropertyInfo(alias="resetsHoursEachYear")]
    """
    Indicates whether the employee's vacation hours reset to zero at the beginning
    of the new year.
    """


class EmployeePayroll(TypedDict, total=False):
    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The employee's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default.
    """

    earnings: Iterable[EmployeePayrollEarning]
    """The employee's earnings."""

    pay_period: Annotated[
        Literal["biweekly", "daily", "monthly", "quarterly", "semimonthly", "weekly", "yearly"],
        PropertyInfo(alias="payPeriod"),
    ]
    """How frequently this employee is paid (e.g., weekly, biweekly, monthly).

    This determines the schedule for generating paychecks.
    """

    sick_hours: Annotated[EmployeePayrollSickHours, PropertyInfo(alias="sickHours")]
    """
    The employee's sick hours, including how sick time is accrued and the total
    hours accrued.
    """

    use_time_data_to_create_paychecks: Annotated[
        Literal["does_not_use_time_data", "not_set", "uses_time_data"],
        PropertyInfo(alias="useTimeDataToCreatePaychecks"),
    ]
    """
    Indicates whether this employee is using time-tracking data to create paychecks.
    """

    vacation_hours: Annotated[EmployeePayrollVacationHours, PropertyInfo(alias="vacationHours")]
    """
    The employee's vacation hours, including how vacation time is accrued and the
    total hours accrued.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import subtotal_item_list_params, subtotal_item_create_params, subtotal_item_update_params
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.subtotal_item import SubtotalItem

__all__ = ["SubtotalItemsResource", "AsyncSubtotalItemsResource"]


class SubtotalItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubtotalItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return SubtotalItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubtotalItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return SubtotalItemsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        barcode: subtotal_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Creates a new subtotal item.

        Args:
          name: The case-insensitive unique name of this subtotal item, unique across all
              subtotal items.

              **NOTE**: Subtotal items do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The subtotal item's barcode.

          description: The subtotal item's description that will appear on sales forms that include
              this item.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_active: Indicates whether this subtotal item is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/subtotal-items",
            body=maybe_transform(
                {
                    "name": name,
                    "barcode": barcode,
                    "description": description,
                    "external_id": external_id,
                    "is_active": is_active,
                },
                subtotal_item_create_params.SubtotalItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Retrieves a subtotal item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the subtotal item to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/subtotal-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        barcode: subtotal_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Updates an existing subtotal item.

        Args:
          id: The QuickBooks-assigned unique identifier of the subtotal item to update.

          revision_number: The current QuickBooks-assigned revision number of the subtotal item object you
              are updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The subtotal item's barcode.

          description: The subtotal item's description that will appear on sales forms that include
              this item.

          is_active: Indicates whether this subtotal item is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive unique name of this subtotal item, unique across all
              subtotal items.

              **NOTE**: Subtotal items do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/subtotal-items/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "barcode": barcode,
                    "description": description,
                    "is_active": is_active,
                    "name": name,
                },
                subtotal_item_update_params.SubtotalItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[SubtotalItem]:
        """Returns a list of subtotal items.

        Use the `cursor` parameter to paginate through
        the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific subtotal items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for subtotal items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for subtotal items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for subtotal items whose `name` is alphabetically greater than or equal
              to this value.

          names: Filter for specific subtotal items by their name(s), case-insensitive. Like
              `id`, `name` is a unique identifier for a subtotal item.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for subtotal items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for subtotal items whose `name` is alphabetically less than or equal to
              this value.

          status: Filter for subtotal items that are active, inactive, or both.

          updated_after: Filter for subtotal items updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for subtotal items updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/subtotal-items",
            page=SyncCursorPage[SubtotalItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    subtotal_item_list_params.SubtotalItemListParams,
                ),
            ),
            model=SubtotalItem,
        )


class AsyncSubtotalItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubtotalItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubtotalItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubtotalItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncSubtotalItemsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        conductor_end_user_id: str,
        barcode: subtotal_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Creates a new subtotal item.

        Args:
          name: The case-insensitive unique name of this subtotal item, unique across all
              subtotal items.

              **NOTE**: Subtotal items do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The subtotal item's barcode.

          description: The subtotal item's description that will appear on sales forms that include
              this item.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          is_active: Indicates whether this subtotal item is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/subtotal-items",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "barcode": barcode,
                    "description": description,
                    "external_id": external_id,
                    "is_active": is_active,
                },
                subtotal_item_create_params.SubtotalItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Retrieves a subtotal item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the subtotal item to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/subtotal-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        barcode: subtotal_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubtotalItem:
        """
        Updates an existing subtotal item.

        Args:
          id: The QuickBooks-assigned unique identifier of the subtotal item to update.

          revision_number: The current QuickBooks-assigned revision number of the subtotal item object you
              are updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The subtotal item's barcode.

          description: The subtotal item's description that will appear on sales forms that include
              this item.

          is_active: Indicates whether this subtotal item is active. Inactive objects are typically
              hidden from views and reports in QuickBooks. Defaults to `true`.

          name: The case-insensitive unique name of this subtotal item, unique across all
              subtotal items.

              **NOTE**: Subtotal items do not have a `fullName` field because they are not
              hierarchical objects, which is why `name` is unique for them but not for objects
              that have parents.

              Maximum length: 31 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/subtotal-items/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "barcode": barcode,
                    "description": description,
                    "is_active": is_active,
                    "name": name,
                },
                subtotal_item_update_params.SubtotalItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubtotalItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        names: List[str] | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[SubtotalItem, AsyncCursorPage[SubtotalItem]]:
        """Returns a list of subtotal items.

        Use the `cursor` parameter to paginate through
        the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific subtotal items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for subtotal items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for subtotal items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for subtotal items whose `name` is alphabetically greater than or equal
              to this value.

          names: Filter for specific subtotal items by their name(s), case-insensitive. Like
              `id`, `name` is a unique identifier for a subtotal item.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          name_starts_with: Filter for subtotal items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for subtotal items whose `name` is alphabetically less than or equal to
              this value.

          status: Filter for subtotal items that are active, inactive, or both.

          updated_after: Filter for subtotal items updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for subtotal items updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/subtotal-items",
            page=AsyncCursorPage[SubtotalItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "names": names,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    subtotal_item_list_params.SubtotalItemListParams,
                ),
            ),
            model=SubtotalItem,
        )


class SubtotalItemsResourceWithRawResponse:
    def __init__(self, subtotal_items: SubtotalItemsResource) -> None:
        self._subtotal_items = subtotal_items

        self.create = to_raw_response_wrapper(
            subtotal_items.create,
        )
        self.retrieve = to_raw_response_wrapper(
            subtotal_items.retrieve,
        )
        self.update = to_raw_response_wrapper(
            subtotal_items.update,
        )
        self.list = to_raw_response_wrapper(
            subtotal_items.list,
        )


class AsyncSubtotalItemsResourceWithRawResponse:
    def __init__(self, subtotal_items: AsyncSubtotalItemsResource) -> None:
        self._subtotal_items = subtotal_items

        self.create = async_to_raw_response_wrapper(
            subtotal_items.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            subtotal_items.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            subtotal_items.update,
        )
        self.list = async_to_raw_response_wrapper(
            subtotal_items.list,
        )


class SubtotalItemsResourceWithStreamingResponse:
    def __init__(self, subtotal_items: SubtotalItemsResource) -> None:
        self._subtotal_items = subtotal_items

        self.create = to_streamed_response_wrapper(
            subtotal_items.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            subtotal_items.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            subtotal_items.update,
        )
        self.list = to_streamed_response_wrapper(
            subtotal_items.list,
        )


class AsyncSubtotalItemsResourceWithStreamingResponse:
    def __init__(self, subtotal_items: AsyncSubtotalItemsResource) -> None:
        self._subtotal_items = subtotal_items

        self.create = async_to_streamed_response_wrapper(
            subtotal_items.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            subtotal_items.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            subtotal_items.update,
        )
        self.list = async_to_streamed_response_wrapper(
            subtotal_items.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PayrollWageItem", "ExpenseAccount"]


class ExpenseAccount(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class PayrollWageItem(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this payroll wage item.

    This ID is unique across all payroll wage items but not across different
    QuickBooks object types.
    """

    created_at: str = FieldInfo(alias="createdAt")
    """
    The date and time when this payroll wage item was created, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    expense_account: ExpenseAccount = FieldInfo(alias="expenseAccount")
    """
    The expense account used to track wage expenses paid through this payroll wage
    item.
    """

    is_active: bool = FieldInfo(alias="isActive")
    """Indicates whether this payroll wage item is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    name: str
    """
    The case-insensitive unique name of this payroll wage item, unique across all
    payroll wage items.

    **NOTE**: Payroll wage items do not have a `fullName` field because they are not
    hierarchical objects, which is why `name` is unique for them but not for objects
    that have parents.
    """

    object_type: Literal["qbd_payroll_wage_item"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_payroll_wage_item"`."""

    revision_number: str = FieldInfo(alias="revisionNumber")
    """
    The current QuickBooks-assigned revision number of this payroll wage item
    object, which changes each time the object is modified. When updating this
    object, you must provide the most recent `revisionNumber` to ensure you're
    working with the latest data; otherwise, the update will return an error.
    """

    updated_at: str = FieldInfo(alias="updatedAt")
    """
    The date and time when this payroll wage item was last updated, in ISO 8601
    format (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time
    zone in QuickBooks.
    """

    wage_type: Literal[
        "bonus",
        "commission",
        "hourly_overtime",
        "hourly_regular",
        "hourly_sick",
        "hourly_vacation",
        "salary_regular",
        "salary_sick",
        "salary_vacation",
    ] = FieldInfo(alias="wageType")
    """
    Categorizes how this payroll wage item calculates pay - can be hourly (regular,
    overtime, sick, or vacation), salary (regular, sick, or vacation), bonus, or
    commission based.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import date
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import (
    time_tracking_activity_list_params,
    time_tracking_activity_create_params,
    time_tracking_activity_update_params,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.time_tracking_activity import TimeTrackingActivity
from ...types.qbd.time_tracking_activity_delete_response import TimeTrackingActivityDeleteResponse

__all__ = ["TimeTrackingActivitiesResource", "AsyncTimeTrackingActivitiesResource"]


class TimeTrackingActivitiesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TimeTrackingActivitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return TimeTrackingActivitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TimeTrackingActivitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return TimeTrackingActivitiesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        duration: str,
        entity_id: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        billing_status: Literal["billable", "has_been_billed", "not_billable"] | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        payroll_wage_item_id: str | NotGiven = NOT_GIVEN,
        service_item_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Creates a new time tracking activity.

        Args:
          duration: The time spent performing the service during this time tracking activity, in ISO
              8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
              represented as PT1H30M.

              **NOTE**: Although seconds can be specified when creating a time tracking
              activity, they are not returned in responses since QuickBooks Desktop's UI does
              not display seconds.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          entity_id: The employee, vendor, or person on QuickBooks's "Other Names" list whose time is
              being tracked in this time tracking activity. This cannot refer to a customer -
              use the `customer` field to associate a customer or customer-job with this time
              tracking activity.

          transaction_date: The date of this time tracking activity, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          billing_status: The billing status of this time tracking activity.

              **IMPORTANT**: When this field is set to "billable" for time tracking
              activities, both `customer` and `serviceItem` are required so that an invoice
              can be created.

          class_id: The time tracking activity's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          customer_id: The customer or customer-job to which this time tracking activity could be
              billed. If `billingStatus` is set to "billable", this field is required.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          note: A note or comment about this time tracking activity.

          payroll_wage_item_id: The payroll wage item (e.g., Regular Pay, Overtime Pay) to use for this time
              tracking activity. This field can only be used for time tracking if: (1) the
              person specified in `entity` is an employee in QuickBooks, and (2) the "Use time
              data to create paychecks" preference is enabled in their payroll settings.

          service_item_id: The type of service performed during this time tracking activity, referring to
              billable or purchasable services such as specialized labor, consulting hours,
              and professional fees.

              **NOTE**: This field is not required if no `customer` is specified. However, if
              `billingStatus` is set to "billable", both this field and `customer` are
              required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/time-tracking-activities",
            body=maybe_transform(
                {
                    "duration": duration,
                    "entity_id": entity_id,
                    "transaction_date": transaction_date,
                    "billing_status": billing_status,
                    "class_id": class_id,
                    "customer_id": customer_id,
                    "external_id": external_id,
                    "note": note,
                    "payroll_wage_item_id": payroll_wage_item_id,
                    "service_item_id": service_item_id,
                },
                time_tracking_activity_create_params.TimeTrackingActivityCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Retrieves a time tracking activity by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    def update(
        self,
        id: str,
        *,
        duration: str,
        entity_id: str,
        revision_number: str,
        conductor_end_user_id: str,
        billing_status: Literal["billable", "has_been_billed", "not_billable"] | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        payroll_wage_item_id: str | NotGiven = NOT_GIVEN,
        service_item_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Updates an existing time tracking activity.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              update.

          duration: The time spent performing the service during this time tracking activity, in ISO
              8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
              represented as PT1H30M.

              **NOTE**: Although seconds can be specified when creating a time tracking
              activity, they are not returned in responses since QuickBooks Desktop's UI does
              not display seconds.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          entity_id: The employee, vendor, or person on QuickBooks's "Other Names" list whose time is
              being tracked in this time tracking activity. This cannot refer to a customer -
              use the `customer` field to associate a customer or customer-job with this time
              tracking activity.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          revision_number: The current QuickBooks-assigned revision number of the time tracking activity
              object you are updating, which you can get by fetching the object first. Provide
              the most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          billing_status: The billing status of this time tracking activity.

              **IMPORTANT**: When this field is set to "billable" for time tracking
              activities, both `customer` and `serviceItem` are required so that an invoice
              can be created.

          class_id: The time tracking activity's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          customer_id: The customer or customer-job to which this time tracking activity could be
              billed. If `billingStatus` is set to "billable", this field is required.

          note: A note or comment about this time tracking activity.

          payroll_wage_item_id: The payroll wage item (e.g., Regular Pay, Overtime Pay) to use for this time
              tracking activity. This field can only be used for time tracking if: (1) the
              person specified in `entity` is an employee in QuickBooks, and (2) the "Use time
              data to create paychecks" preference is enabled in their payroll settings.

          service_item_id: The type of service performed during this time tracking activity, referring to
              billable or purchasable services such as specialized labor, consulting hours,
              and professional fees.

              **NOTE**: This field is not required if no `customer` is specified. However, if
              `billingStatus` is set to "billable", both this field and `customer` are
              required.

          transaction_date: The date of this time tracking activity, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            body=maybe_transform(
                {
                    "duration": duration,
                    "entity_id": entity_id,
                    "revision_number": revision_number,
                    "billing_status": billing_status,
                    "class_id": class_id,
                    "customer_id": customer_id,
                    "note": note,
                    "payroll_wage_item_id": payroll_wage_item_id,
                    "service_item_id": service_item_id,
                    "transaction_date": transaction_date,
                },
                time_tracking_activity_update_params.TimeTrackingActivityUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        entity_ids: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[TimeTrackingActivity]:
        """Returns a list of time tracking activities.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          entity_ids: Filter for time tracking activities tracking the time of these employees,
              vendors, or persons on QuickBooks's "Other Names" list.

          ids: Filter for specific time tracking activities by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          transaction_date_from: Filter for time tracking activities whose `date` field is on or after this date,
              in ISO 8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for time tracking activities whose `date` field is on or before this
              date, in ISO 8601 format (YYYY-MM-DD).

          updated_after: Filter for time tracking activities updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

          updated_before: Filter for time tracking activities updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/time-tracking-activities",
            page=SyncCursorPage[TimeTrackingActivity],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "entity_ids": entity_ids,
                        "ids": ids,
                        "limit": limit,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    time_tracking_activity_list_params.TimeTrackingActivityListParams,
                ),
            ),
            model=TimeTrackingActivity,
        )

    def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivityDeleteResponse:
        """Permanently deletes a a time tracking activity.

        The deletion will fail if the
        time tracking activity is currently in use or has any linked transactions that
        are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._delete(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivityDeleteResponse,
        )


class AsyncTimeTrackingActivitiesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTimeTrackingActivitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTimeTrackingActivitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTimeTrackingActivitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncTimeTrackingActivitiesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        duration: str,
        entity_id: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        billing_status: Literal["billable", "has_been_billed", "not_billable"] | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        payroll_wage_item_id: str | NotGiven = NOT_GIVEN,
        service_item_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Creates a new time tracking activity.

        Args:
          duration: The time spent performing the service during this time tracking activity, in ISO
              8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
              represented as PT1H30M.

              **NOTE**: Although seconds can be specified when creating a time tracking
              activity, they are not returned in responses since QuickBooks Desktop's UI does
              not display seconds.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          entity_id: The employee, vendor, or person on QuickBooks's "Other Names" list whose time is
              being tracked in this time tracking activity. This cannot refer to a customer -
              use the `customer` field to associate a customer or customer-job with this time
              tracking activity.

          transaction_date: The date of this time tracking activity, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          billing_status: The billing status of this time tracking activity.

              **IMPORTANT**: When this field is set to "billable" for time tracking
              activities, both `customer` and `serviceItem` are required so that an invoice
              can be created.

          class_id: The time tracking activity's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          customer_id: The customer or customer-job to which this time tracking activity could be
              billed. If `billingStatus` is set to "billable", this field is required.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          note: A note or comment about this time tracking activity.

          payroll_wage_item_id: The payroll wage item (e.g., Regular Pay, Overtime Pay) to use for this time
              tracking activity. This field can only be used for time tracking if: (1) the
              person specified in `entity` is an employee in QuickBooks, and (2) the "Use time
              data to create paychecks" preference is enabled in their payroll settings.

          service_item_id: The type of service performed during this time tracking activity, referring to
              billable or purchasable services such as specialized labor, consulting hours,
              and professional fees.

              **NOTE**: This field is not required if no `customer` is specified. However, if
              `billingStatus` is set to "billable", both this field and `customer` are
              required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/time-tracking-activities",
            body=await async_maybe_transform(
                {
                    "duration": duration,
                    "entity_id": entity_id,
                    "transaction_date": transaction_date,
                    "billing_status": billing_status,
                    "class_id": class_id,
                    "customer_id": customer_id,
                    "external_id": external_id,
                    "note": note,
                    "payroll_wage_item_id": payroll_wage_item_id,
                    "service_item_id": service_item_id,
                },
                time_tracking_activity_create_params.TimeTrackingActivityCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Retrieves a time tracking activity by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    async def update(
        self,
        id: str,
        *,
        duration: str,
        entity_id: str,
        revision_number: str,
        conductor_end_user_id: str,
        billing_status: Literal["billable", "has_been_billed", "not_billable"] | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        customer_id: str | NotGiven = NOT_GIVEN,
        note: str | NotGiven = NOT_GIVEN,
        payroll_wage_item_id: str | NotGiven = NOT_GIVEN,
        service_item_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivity:
        """
        Updates an existing time tracking activity.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              update.

          duration: The time spent performing the service during this time tracking activity, in ISO
              8601 format for time intervals (PTnHnMnS). For example, 1 hour and 30 minutes is
              represented as PT1H30M.

              **NOTE**: Although seconds can be specified when creating a time tracking
              activity, they are not returned in responses since QuickBooks Desktop's UI does
              not display seconds.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          entity_id: The employee, vendor, or person on QuickBooks's "Other Names" list whose time is
              being tracked in this time tracking activity. This cannot refer to a customer -
              use the `customer` field to associate a customer or customer-job with this time
              tracking activity.

              **IMPORTANT**: This field is required for updating time tracking activities,
              even if the field is not being modified, because of a bug in QuickBooks itself.

          revision_number: The current QuickBooks-assigned revision number of the time tracking activity
              object you are updating, which you can get by fetching the object first. Provide
              the most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          billing_status: The billing status of this time tracking activity.

              **IMPORTANT**: When this field is set to "billable" for time tracking
              activities, both `customer` and `serviceItem` are required so that an invoice
              can be created.

          class_id: The time tracking activity's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          customer_id: The customer or customer-job to which this time tracking activity could be
              billed. If `billingStatus` is set to "billable", this field is required.

          note: A note or comment about this time tracking activity.

          payroll_wage_item_id: The payroll wage item (e.g., Regular Pay, Overtime Pay) to use for this time
              tracking activity. This field can only be used for time tracking if: (1) the
              person specified in `entity` is an employee in QuickBooks, and (2) the "Use time
              data to create paychecks" preference is enabled in their payroll settings.

          service_item_id: The type of service performed during this time tracking activity, referring to
              billable or purchasable services such as specialized labor, consulting hours,
              and professional fees.

              **NOTE**: This field is not required if no `customer` is specified. However, if
              `billingStatus` is set to "billable", both this field and `customer` are
              required.

          transaction_date: The date of this time tracking activity, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            body=await async_maybe_transform(
                {
                    "duration": duration,
                    "entity_id": entity_id,
                    "revision_number": revision_number,
                    "billing_status": billing_status,
                    "class_id": class_id,
                    "customer_id": customer_id,
                    "note": note,
                    "payroll_wage_item_id": payroll_wage_item_id,
                    "service_item_id": service_item_id,
                    "transaction_date": transaction_date,
                },
                time_tracking_activity_update_params.TimeTrackingActivityUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivity,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        cursor: str | NotGiven = NOT_GIVEN,
        entity_ids: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TimeTrackingActivity, AsyncCursorPage[TimeTrackingActivity]]:
        """Returns a list of time tracking activities.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          entity_ids: Filter for time tracking activities tracking the time of these employees,
              vendors, or persons on QuickBooks's "Other Names" list.

          ids: Filter for specific time tracking activities by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          transaction_date_from: Filter for time tracking activities whose `date` field is on or after this date,
              in ISO 8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for time tracking activities whose `date` field is on or before this
              date, in ISO 8601 format (YYYY-MM-DD).

          updated_after: Filter for time tracking activities updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

          updated_before: Filter for time tracking activities updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/time-tracking-activities",
            page=AsyncCursorPage[TimeTrackingActivity],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "entity_ids": entity_ids,
                        "ids": ids,
                        "limit": limit,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    time_tracking_activity_list_params.TimeTrackingActivityListParams,
                ),
            ),
            model=TimeTrackingActivity,
        )

    async def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TimeTrackingActivityDeleteResponse:
        """Permanently deletes a a time tracking activity.

        The deletion will fail if the
        time tracking activity is currently in use or has any linked transactions that
        are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the time tracking activity to
              delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._delete(
            f"/quickbooks-desktop/time-tracking-activities/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TimeTrackingActivityDeleteResponse,
        )


class TimeTrackingActivitiesResourceWithRawResponse:
    def __init__(self, time_tracking_activities: TimeTrackingActivitiesResource) -> None:
        self._time_tracking_activities = time_tracking_activities

        self.create = to_raw_response_wrapper(
            time_tracking_activities.create,
        )
        self.retrieve = to_raw_response_wrapper(
            time_tracking_activities.retrieve,
        )
        self.update = to_raw_response_wrapper(
            time_tracking_activities.update,
        )
        self.list = to_raw_response_wrapper(
            time_tracking_activities.list,
        )
        self.delete = to_raw_response_wrapper(
            time_tracking_activities.delete,
        )


class AsyncTimeTrackingActivitiesResourceWithRawResponse:
    def __init__(self, time_tracking_activities: AsyncTimeTrackingActivitiesResource) -> None:
        self._time_tracking_activities = time_tracking_activities

        self.create = async_to_raw_response_wrapper(
            time_tracking_activities.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            time_tracking_activities.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            time_tracking_activities.update,
        )
        self.list = async_to_raw_response_wrapper(
            time_tracking_activities.list,
        )
        self.delete = async_to_raw_response_wrapper(
            time_tracking_activities.delete,
        )


class TimeTrackingActivitiesResourceWithStreamingResponse:
    def __init__(self, time_tracking_activities: TimeTrackingActivitiesResource) -> None:
        self._time_tracking_activities = time_tracking_activities

        self.create = to_streamed_response_wrapper(
            time_tracking_activities.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            time_tracking_activities.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            time_tracking_activities.update,
        )
        self.list = to_streamed_response_wrapper(
            time_tracking_activities.list,
        )
        self.delete = to_streamed_response_wrapper(
            time_tracking_activities.delete,
        )


class AsyncTimeTrackingActivitiesResourceWithStreamingResponse:
    def __init__(self, time_tracking_activities: AsyncTimeTrackingActivitiesResource) -> None:
        self._time_tracking_activities = time_tracking_activities

        self.create = async_to_streamed_response_wrapper(
            time_tracking_activities.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            time_tracking_activities.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            time_tracking_activities.update,
        )
        self.list = async_to_streamed_response_wrapper(
            time_tracking_activities.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            time_tracking_activities.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "SalesReceiptCreateParams",
    "BillingAddress",
    "CreditCardTransaction",
    "CreditCardTransactionRequest",
    "CreditCardTransactionResponse",
    "LineGroup",
    "LineGroupCustomField",
    "Line",
    "LineCreditCardTransaction",
    "LineCreditCardTransactionRequest",
    "LineCreditCardTransactionResponse",
    "LineCustomField",
    "ShippingAddress",
]


class SalesReceiptCreateParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """
    The customer or customer-job to which the payment for this sales receipt is
    credited.
    """

    transaction_date: Required[Annotated[Union[str, date], PropertyInfo(alias="transactionDate", format="iso8601")]]
    """The date of this sales receipt, in ISO 8601 format (YYYY-MM-DD)."""

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    billing_address: Annotated[BillingAddress, PropertyInfo(alias="billingAddress")]
    """The sales receipt's billing address."""

    check_number: Annotated[str, PropertyInfo(alias="checkNumber")]
    """The check number of a check received for this sales receipt."""

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The sales receipt's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. A class defined here is automatically used in this sales receipt's line
    items unless overridden at the line item level.
    """

    credit_card_transaction: Annotated[CreditCardTransaction, PropertyInfo(alias="creditCardTransaction")]
    """
    The credit card transaction data for this sales receipt's payment when using
    QuickBooks Merchant Services (QBMS). If specifying this field, you must also
    specify the `paymentMethod` field.
    """

    customer_message_id: Annotated[str, PropertyInfo(alias="customerMessageId")]
    """The message to display to the customer on the sales receipt."""

    deposit_to_account_id: Annotated[str, PropertyInfo(alias="depositToAccountId")]
    """
    The account where the funds for this sales receipt will be or have been
    deposited.
    """

    document_template_id: Annotated[str, PropertyInfo(alias="documentTemplateId")]
    """
    The predefined template in QuickBooks that determines the layout and formatting
    for this sales receipt when printed or displayed.
    """

    due_date: Annotated[Union[str, date], PropertyInfo(alias="dueDate", format="iso8601")]
    """
    The date by which this sales receipt must be paid, in ISO 8601 format
    (YYYY-MM-DD).

    **NOTE**: For sales receipts, this field is often `null` because sales receipts
    are generally used for point-of-sale payments, where full payment is received at
    the time of purchase.
    """

    exchange_rate: Annotated[float, PropertyInfo(alias="exchangeRate")]
    """
    The market exchange rate between this sales receipt's currency and the home
    currency in QuickBooks at the time of this transaction. Represented as a decimal
    value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).
    """

    external_id: Annotated[str, PropertyInfo(alias="externalId")]
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.

    **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
    QuickBooks will return an error.
    """

    is_pending: Annotated[bool, PropertyInfo(alias="isPending")]
    """Indicates whether this sales receipt has not been completed."""

    is_queued_for_email: Annotated[bool, PropertyInfo(alias="isQueuedForEmail")]
    """
    Indicates whether this sales receipt is included in the queue of documents for
    QuickBooks to email to the customer.
    """

    is_queued_for_print: Annotated[bool, PropertyInfo(alias="isQueuedForPrint")]
    """
    Indicates whether this sales receipt is included in the queue of documents for
    QuickBooks to print.
    """

    line_groups: Annotated[Iterable[LineGroup], PropertyInfo(alias="lineGroups")]
    """
    The sales receipt's line item groups, each representing a predefined set of
    related items.

    **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
    sales receipt.
    """

    lines: Iterable[Line]
    """
    The sales receipt's line items, each representing a single product or service
    sold.

    **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
    sales receipt.
    """

    memo: str
    """
    A memo or note for this sales receipt that appears in reports, but not on the
    sales receipt.
    """

    other_custom_field: Annotated[str, PropertyInfo(alias="otherCustomField")]
    """
    A built-in custom field for additional information specific to this sales
    receipt. Unlike the user-defined fields in the `customFields` array, this is a
    standard QuickBooks field that exists for all sales receipts for convenience.
    Developers often use this field for tracking information that doesn't fit into
    other standard QuickBooks fields. Unlike `otherCustomField1` and
    `otherCustomField2`, which are line item fields, this exists at the transaction
    level. Hidden by default in the QuickBooks UI.
    """

    payment_method_id: Annotated[str, PropertyInfo(alias="paymentMethodId")]
    """The sales receipt's payment method (e.g., cash, check, credit card).

    **NOTE**: If this sales receipt contains credit card transaction data supplied
    from QuickBooks Merchant Services (QBMS) transaction responses, you must specify
    a credit card payment method (e.g., "Visa", "MasterCard", etc.).
    """

    ref_number: Annotated[str, PropertyInfo(alias="refNumber")]
    """
    The case-sensitive user-defined reference number for this sales receipt, which
    can be used to identify the transaction in QuickBooks. This value is not
    required to be unique and can be arbitrarily changed by the QuickBooks user.
    When left blank in this create request, this field will be left blank in
    QuickBooks (i.e., it does _not_ auto-increment).
    """

    sales_representative_id: Annotated[str, PropertyInfo(alias="salesRepresentativeId")]
    """The sales receipt's sales representative.

    Sales representatives can be employees, vendors, or other names in QuickBooks.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The sales-tax code for this sales receipt, determining whether it is taxable or
    non-taxable. This can be overridden at the transaction-line level.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    sales_tax_item_id: Annotated[str, PropertyInfo(alias="salesTaxItemId")]
    """
    The sales-tax item used to calculate the actual tax amount for this sales
    receipt's transactions by applying a specific tax rate collected for a single
    tax agency. Unlike `salesTaxCode`, which only indicates general taxability, this
    field drives the actual tax calculation and reporting.

    For sales receipts, while using this field to specify a single tax item/group
    that applies uniformly is recommended, complex tax scenarios may require
    alternative approaches. In such cases, you can set this field to a 0% tax item
    (conventionally named "Tax Calculated On Invoice") and handle tax calculations
    through line items instead. When using line items for taxes, note that only
    individual tax items (not tax groups) can be used, subtotals can help apply a
    tax to multiple items but only the first tax line after a subtotal is calculated
    automatically (subsequent tax lines require manual amounts), and the rate column
    will always display the actual tax amount rather than the rate percentage.
    """

    shipment_origin: Annotated[str, PropertyInfo(alias="shipmentOrigin")]
    """
    The origin location from where the product associated with this sales receipt is
    shipped. This is the point at which ownership and liability for goods transfer
    from seller to buyer. Internally, QuickBooks uses the term "FOB" for this field,
    which stands for "freight on board". This field is informational and has no
    accounting implications.
    """

    shipping_address: Annotated[ShippingAddress, PropertyInfo(alias="shippingAddress")]
    """The sales receipt's shipping address."""

    shipping_date: Annotated[Union[str, date], PropertyInfo(alias="shippingDate", format="iso8601")]
    """
    The date when the products or services for this sales receipt were shipped or
    are expected to be shipped, in ISO 8601 format (YYYY-MM-DD).
    """

    shipping_method_id: Annotated[str, PropertyInfo(alias="shippingMethodId")]
    """
    The shipping method used for this sales receipt, such as standard mail or
    overnight delivery.
    """


class BillingAddress(TypedDict, total=False):
    city: str
    """The city, district, suburb, town, or village name of the address.

    Maximum length: 31 characters.
    """

    country: str
    """The country name of the address."""

    line1: str
    """The first line of the address (e.g., street, PO Box, or company name).

    Maximum length: 41 characters.
    """

    line2: str
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).

    Maximum length: 41 characters.
    """

    line3: str
    """The third line of the address, if needed.

    Maximum length: 41 characters.
    """

    line4: str
    """The fourth line of the address, if needed.

    Maximum length: 41 characters.
    """

    line5: str
    """The fifth line of the address, if needed.

    Maximum length: 41 characters.
    """

    note: str
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The postal code or ZIP code of the address.

    Maximum length: 13 characters.
    """

    state: str
    """The state, county, province, or region name of the address.

    Maximum length: 21 characters.
    """


class CreditCardTransactionRequest(TypedDict, total=False):
    expiration_month: Required[Annotated[float, PropertyInfo(alias="expirationMonth")]]
    """The month when the credit card expires."""

    expiration_year: Required[Annotated[float, PropertyInfo(alias="expirationYear")]]
    """The year when the credit card expires."""

    name: Required[str]
    """The cardholder's name on the card."""

    number: Required[str]
    """The credit card number. Must be masked with lower case "x" and no dashes."""

    address: str
    """The card's billing address."""

    commercial_card_code: Annotated[str, PropertyInfo(alias="commercialCardCode")]
    """
    The commercial card code identifies the type of business credit card being used
    (purchase, corporate, or business) for Visa and Mastercard transactions only.
    When provided, this code may qualify the transaction for lower processing fees
    compared to the standard rates that apply when no code is specified.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The card's billing address ZIP or postal code."""

    transaction_mode: Annotated[Literal["card_not_present", "card_present"], PropertyInfo(alias="transactionMode")]
    """
    Indicates whether this credit card transaction came from a card swipe
    (`card_present`) or not (`card_not_present`).
    """

    transaction_type: Annotated[
        Literal["authorization", "capture", "charge", "refund", "voice_authorization"],
        PropertyInfo(alias="transactionType"),
    ]
    """The QBMS transaction type from which the current transaction data originated."""


class CreditCardTransactionResponse(TypedDict, total=False):
    credit_card_transaction_id: Required[Annotated[str, PropertyInfo(alias="creditCardTransactionId")]]
    """
    The ID returned from the credit card processor for this credit card transaction.
    """

    merchant_account_number: Required[Annotated[str, PropertyInfo(alias="merchantAccountNumber")]]
    """
    The QBMS account number of the merchant who is running this transaction using
    the customer's credit card.
    """

    payment_status: Required[Annotated[Literal["completed", "unknown"], PropertyInfo(alias="paymentStatus")]]
    """
    Indicates whether this credit card transaction is known to have been
    successfully processed by the card issuer.
    """

    status_code: Required[Annotated[float, PropertyInfo(alias="statusCode")]]
    """
    The status code returned in the original QBMS transaction response for this
    credit card transaction.
    """

    status_message: Required[Annotated[str, PropertyInfo(alias="statusMessage")]]
    """
    The status message returned in the original QBMS transaction response for this
    credit card transaction.
    """

    transaction_authorized_at: Required[Annotated[str, PropertyInfo(alias="transactionAuthorizedAt")]]
    """
    The date and time when the credit card processor authorized this credit card
    transaction.
    """

    authorization_code: Annotated[str, PropertyInfo(alias="authorizationCode")]
    """
    The authorization code returned from the credit card processor to indicate that
    this charge will be paid by the card issuer.
    """

    avs_street_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsStreetStatus")]
    """
    Indicates whether the street address supplied in the transaction request matches
    the customer's address on file at the card issuer.
    """

    avs_zip_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsZipStatus")]
    """
    Indicates whether the customer postal ZIP code supplied in the transaction
    request matches the customer's postal code recognized at the card issuer.
    """

    card_security_code_match: Annotated[
        Literal["fail", "not_available", "pass"], PropertyInfo(alias="cardSecurityCodeMatch")
    ]
    """
    Indicates whether the card security code supplied in the transaction request
    matches the card security code recognized for that credit card number at the
    card issuer.
    """

    client_transaction_id: Annotated[str, PropertyInfo(alias="clientTransactionId")]
    """
    A value returned from QBMS transactions for future use by the QuickBooks
    Reconciliation feature.
    """

    payment_grouping_code: Annotated[float, PropertyInfo(alias="paymentGroupingCode")]
    """
    An internal code returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    recon_batch_id: Annotated[str, PropertyInfo(alias="reconBatchId")]
    """
    An internal ID returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    transaction_authorization_stamp: Annotated[float, PropertyInfo(alias="transactionAuthorizationStamp")]
    """
    An internal value for this credit card transaction, needed for the QuickBooks
    reconciliation feature.
    """


class CreditCardTransaction(TypedDict, total=False):
    request: CreditCardTransactionRequest
    """
    The transaction request data originally supplied for this credit card
    transaction when using QuickBooks Merchant Services (QBMS).
    """

    response: CreditCardTransactionResponse
    """
    The transaction response data for this credit card transaction when using
    QuickBooks Merchant Services (QBMS).
    """


class LineGroupCustomField(TypedDict, total=False):
    name: Required[str]
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: Required[Annotated[str, PropertyInfo(alias="ownerId")]]
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    value: Required[str]
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class LineGroup(TypedDict, total=False):
    item_group_id: Required[Annotated[str, PropertyInfo(alias="itemGroupId")]]
    """
    The sales receipt line group's item group, representing a predefined set of
    items bundled because they are commonly purchased together or grouped for faster
    entry.
    """

    custom_fields: Annotated[Iterable[LineGroupCustomField], PropertyInfo(alias="customFields")]
    """
    The custom fields for the sales receipt line group object, added as user-defined
    data extensions, not included in the standard QuickBooks object.
    """

    inventory_site_id: Annotated[str, PropertyInfo(alias="inventorySiteId")]
    """
    The site location where inventory for the item group associated with this sales
    receipt line group is stored.
    """

    inventory_site_location_id: Annotated[str, PropertyInfo(alias="inventorySiteLocationId")]
    """
    The specific location (e.g., bin or shelf) within the inventory site where the
    item group associated with this sales receipt line group is stored.
    """

    quantity: float
    """The quantity of the item group associated with this sales receipt line group.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item group is a discount item
    group.
    """

    unit_of_measure: Annotated[str, PropertyInfo(alias="unitOfMeasure")]
    """The unit-of-measure used for the `quantity` in this sales receipt line group.

    Must be a valid unit within the item's available units of measure.
    """


class LineCreditCardTransactionRequest(TypedDict, total=False):
    expiration_month: Required[Annotated[float, PropertyInfo(alias="expirationMonth")]]
    """The month when the credit card expires."""

    expiration_year: Required[Annotated[float, PropertyInfo(alias="expirationYear")]]
    """The year when the credit card expires."""

    name: Required[str]
    """The cardholder's name on the card."""

    number: Required[str]
    """The credit card number. Must be masked with lower case "x" and no dashes."""

    address: str
    """The card's billing address."""

    commercial_card_code: Annotated[str, PropertyInfo(alias="commercialCardCode")]
    """
    The commercial card code identifies the type of business credit card being used
    (purchase, corporate, or business) for Visa and Mastercard transactions only.
    When provided, this code may qualify the transaction for lower processing fees
    compared to the standard rates that apply when no code is specified.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The card's billing address ZIP or postal code."""

    transaction_mode: Annotated[Literal["card_not_present", "card_present"], PropertyInfo(alias="transactionMode")]
    """
    Indicates whether this credit card transaction came from a card swipe
    (`card_present`) or not (`card_not_present`).
    """

    transaction_type: Annotated[
        Literal["authorization", "capture", "charge", "refund", "voice_authorization"],
        PropertyInfo(alias="transactionType"),
    ]
    """The QBMS transaction type from which the current transaction data originated."""


class LineCreditCardTransactionResponse(TypedDict, total=False):
    credit_card_transaction_id: Required[Annotated[str, PropertyInfo(alias="creditCardTransactionId")]]
    """
    The ID returned from the credit card processor for this credit card transaction.
    """

    merchant_account_number: Required[Annotated[str, PropertyInfo(alias="merchantAccountNumber")]]
    """
    The QBMS account number of the merchant who is running this transaction using
    the customer's credit card.
    """

    payment_status: Required[Annotated[Literal["completed", "unknown"], PropertyInfo(alias="paymentStatus")]]
    """
    Indicates whether this credit card transaction is known to have been
    successfully processed by the card issuer.
    """

    status_code: Required[Annotated[float, PropertyInfo(alias="statusCode")]]
    """
    The status code returned in the original QBMS transaction response for this
    credit card transaction.
    """

    status_message: Required[Annotated[str, PropertyInfo(alias="statusMessage")]]
    """
    The status message returned in the original QBMS transaction response for this
    credit card transaction.
    """

    transaction_authorized_at: Required[Annotated[str, PropertyInfo(alias="transactionAuthorizedAt")]]
    """
    The date and time when the credit card processor authorized this credit card
    transaction.
    """

    authorization_code: Annotated[str, PropertyInfo(alias="authorizationCode")]
    """
    The authorization code returned from the credit card processor to indicate that
    this charge will be paid by the card issuer.
    """

    avs_street_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsStreetStatus")]
    """
    Indicates whether the street address supplied in the transaction request matches
    the customer's address on file at the card issuer.
    """

    avs_zip_status: Annotated[Literal["fail", "not_available", "pass"], PropertyInfo(alias="avsZipStatus")]
    """
    Indicates whether the customer postal ZIP code supplied in the transaction
    request matches the customer's postal code recognized at the card issuer.
    """

    card_security_code_match: Annotated[
        Literal["fail", "not_available", "pass"], PropertyInfo(alias="cardSecurityCodeMatch")
    ]
    """
    Indicates whether the card security code supplied in the transaction request
    matches the card security code recognized for that credit card number at the
    card issuer.
    """

    client_transaction_id: Annotated[str, PropertyInfo(alias="clientTransactionId")]
    """
    A value returned from QBMS transactions for future use by the QuickBooks
    Reconciliation feature.
    """

    payment_grouping_code: Annotated[float, PropertyInfo(alias="paymentGroupingCode")]
    """
    An internal code returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    recon_batch_id: Annotated[str, PropertyInfo(alias="reconBatchId")]
    """
    An internal ID returned by QuickBooks Merchant Services (QBMS) from the
    transaction request, needed for the QuickBooks reconciliation feature.
    """

    transaction_authorization_stamp: Annotated[float, PropertyInfo(alias="transactionAuthorizationStamp")]
    """
    An internal value for this credit card transaction, needed for the QuickBooks
    reconciliation feature.
    """


class LineCreditCardTransaction(TypedDict, total=False):
    request: LineCreditCardTransactionRequest
    """
    The transaction request data originally supplied for this credit card
    transaction when using QuickBooks Merchant Services (QBMS).
    """

    response: LineCreditCardTransactionResponse
    """
    The transaction response data for this credit card transaction when using
    QuickBooks Merchant Services (QBMS).
    """


class LineCustomField(TypedDict, total=False):
    name: Required[str]
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: Required[Annotated[str, PropertyInfo(alias="ownerId")]]
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    value: Required[str]
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class Line(TypedDict, total=False):
    amount: str
    """The monetary amount of this sales receipt line, represented as a decimal string.

    If both `quantity` and `rate` are specified but not `amount`, QuickBooks will
    use them to calculate `amount`. If `amount`, `rate`, and `quantity` are all
    unspecified, then QuickBooks will calculate `amount` based on a `quantity` of
    `1` and the suggested `rate`. This field cannot be cleared.
    """

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The sales receipt line's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default. If a class is specified for the entire parent transaction, it is
    automatically applied to all sales receipt lines unless overridden here, at the
    transaction line level.
    """

    credit_card_transaction: Annotated[LineCreditCardTransaction, PropertyInfo(alias="creditCardTransaction")]
    """
    The credit card transaction data for this sales receipt line's payment when
    using QuickBooks Merchant Services (QBMS). If specifying this field, you must
    also specify the `paymentMethod` field.
    """

    custom_fields: Annotated[Iterable[LineCustomField], PropertyInfo(alias="customFields")]
    """
    The custom fields for the sales receipt line object, added as user-defined data
    extensions, not included in the standard QuickBooks object.
    """

    description: str
    """A description of this sales receipt line."""

    inventory_site_id: Annotated[str, PropertyInfo(alias="inventorySiteId")]
    """
    The site location where inventory for the item associated with this sales
    receipt line is stored.
    """

    inventory_site_location_id: Annotated[str, PropertyInfo(alias="inventorySiteLocationId")]
    """
    The specific location (e.g., bin or shelf) within the inventory site where the
    item associated with this sales receipt line is stored.
    """

    item_id: Annotated[str, PropertyInfo(alias="itemId")]
    """The item associated with this sales receipt line.

    This can refer to any good or service that the business buys or sells, including
    item types such as a service item, inventory item, or special calculation item
    like a discount item or sales-tax item.
    """

    lot_number: Annotated[str, PropertyInfo(alias="lotNumber")]
    """The lot number of the item associated with this sales receipt line.

    Used for tracking groups of inventory items that are purchased or manufactured
    together.
    """

    other_custom_field1: Annotated[str, PropertyInfo(alias="otherCustomField1")]
    """
    A built-in custom field for additional information specific to this sales
    receipt line. Unlike the user-defined fields in the `customFields` array, this
    is a standard QuickBooks field that exists for all sales receipt lines for
    convenience. Developers often use this field for tracking information that
    doesn't fit into other standard QuickBooks fields. Hidden by default in the
    QuickBooks UI.
    """

    other_custom_field2: Annotated[str, PropertyInfo(alias="otherCustomField2")]
    """
    A second built-in custom field for additional information specific to this sales
    receipt line. Unlike the user-defined fields in the `customFields` array, this
    is a standard QuickBooks field that exists for all sales receipt lines for
    convenience. Like `otherCustomField1`, developers often use this field for
    tracking information that doesn't fit into other standard QuickBooks fields.
    Hidden by default in the QuickBooks UI.
    """

    override_item_account_id: Annotated[str, PropertyInfo(alias="overrideItemAccountId")]
    """
    The account to use for this sales receipt line, overriding the default account
    associated with the item.
    """

    price_level_id: Annotated[str, PropertyInfo(alias="priceLevelId")]
    """The price level applied to this sales receipt line.

    This overrides any price level set on the corresponding customer. The resulting
    sales receipt line will not show this price level, only the final `rate`
    calculated from it.
    """

    price_rule_conflict_strategy: Annotated[
        Literal["base_price", "zero"], PropertyInfo(alias="priceRuleConflictStrategy")
    ]
    """
    Specifies how to resolve price rule conflicts when adding or modifying this
    sales receipt line.
    """

    quantity: float
    """The quantity of the item associated with this sales receipt line.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item is a discount item.
    """

    rate: str
    """The price per unit for this sales receipt line.

    If both `rate` and `amount` are specified, `rate` will be ignored. If both
    `quantity` and `amount` are specified but not `rate`, QuickBooks will use them
    to calculate `rate`. Represented as a decimal string. This field cannot be
    cleared.
    """

    rate_percent: Annotated[str, PropertyInfo(alias="ratePercent")]
    """The price of this sales receipt line expressed as a percentage.

    Typically used for discount or markup items.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The sales-tax code for this sales receipt line, determining whether it is
    taxable or non-taxable. If set, this overrides any sales-tax codes defined on
    the parent transaction or the associated item.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    serial_number: Annotated[str, PropertyInfo(alias="serialNumber")]
    """The serial number of the item associated with this sales receipt line.

    This is used for tracking individual units of serialized inventory items.
    """

    service_date: Annotated[Union[str, date], PropertyInfo(alias="serviceDate", format="iso8601")]
    """
    The date on which the service for this sales receipt line was or will be
    performed, in ISO 8601 format (YYYY-MM-DD). This is particularly relevant for
    service items.
    """

    unit_of_measure: Annotated[str, PropertyInfo(alias="unitOfMeasure")]
    """The unit-of-measure used for the `quantity` in this sales receipt line.

    Must be a valid unit within the item's available units of measure.
    """


class ShippingAddress(TypedDict, total=False):
    city: str
    """The city, district, suburb, town, or village name of the address.

    Maximum length: 31 characters.
    """

    country: str
    """The country name of the address."""

    line1: str
    """The first line of the address (e.g., street, PO Box, or company name).

    Maximum length: 41 characters.
    """

    line2: str
    """
    The second line of the address, if needed (e.g., apartment, suite, unit, or
    building).

    Maximum length: 41 characters.
    """

    line3: str
    """The third line of the address, if needed.

    Maximum length: 41 characters.
    """

    line4: str
    """The fourth line of the address, if needed.

    Maximum length: 41 characters.
    """

    line5: str
    """The fifth line of the address, if needed.

    Maximum length: 41 characters.
    """

    note: str
    """
    A note written at the bottom of the address in the form in which it appears,
    such as the invoice form.
    """

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]
    """The postal code or ZIP code of the address.

    Maximum length: 13 characters.
    """

    state: str
    """The state, county, province, or region name of the address.

    Maximum length: 21 characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from conductor import Conductor, AsyncConductor
from tests.utils import assert_matches_type
from conductor.types import QbdHealthCheckResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestQbd:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_health_check(self, client: Conductor) -> None:
        qbd = client.qbd.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

    @parametrize
    def test_raw_response_health_check(self, client: Conductor) -> None:
        response = client.qbd.with_raw_response.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        qbd = response.parse()
        assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

    @parametrize
    def test_streaming_response_health_check(self, client: Conductor) -> None:
        with client.qbd.with_streaming_response.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            qbd = response.parse()
            assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncQbd:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_health_check(self, async_client: AsyncConductor) -> None:
        qbd = await async_client.qbd.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

    @parametrize
    async def test_raw_response_health_check(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.with_raw_response.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        qbd = await response.parse()
        assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

    @parametrize
    async def test_streaming_response_health_check(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.with_streaming_response.health_check(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            qbd = await response.parse()
            assert_matches_type(QbdHealthCheckResponse, qbd, path=["response"])

        assert cast(Any, response.is_closed) is True
